# These values are "required" to create a Pexip Transcoder within Manager
REQUIRED_CREATE_ARGS = [
    "hostname", 
    "domain", 
    "private_ip", 
    "public_ip", 
    "netmask", 
    "gateway", 
    "node_password", 
    "tls_certificate_subject_name", 
    "system_location_name"
]
